@@ -743,3 +743,63 @@ impl<'a, E: Endianness> From<&'a [u8]> for BitReadStream<'a, E> {
         BitReadStream::new(BitReadBuffer::from(bytes))
     }
 }
+
+#[cfg(feature = "serde")]
+use serde::{de, ser::SerializeStruct, Deserialize, Deserializer, Serialize, Serializer};
+
+#[cfg(feature = "serde")]
+impl<'a, E: Endianness> Serialize for BitReadStream<'a, E> {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        let mut stream = self.clone();
+        let mut data = stream.read_bytes(self.bits_left() / 8).unwrap().to_vec();
+        if stream.bits_left() > 0 {
+            data.push(stream.read_sized(stream.bits_left()).unwrap());
+        }
+
+        let mut s = serializer.serialize_struct("BitReadStream", 3)?;
+        s.serialize_field("data", &data)?;
+        s.serialize_field("bit_length", &self.bit_len())?;
+        s.end()
+    }
+}
+
+#[cfg(feature = "serde")]
+impl<'de, E: Endianness> Deserialize<'de> for BitReadStream<'static, E> {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        #[derive(Deserialize)]
+        struct BitData {
+            data: Vec<u8>,
+            bit_length: usize,
+        }
+
+        let data = BitData::deserialize(deserializer)?;
+        let mut buffer = BitReadBuffer::new_owned(data.data, E::endianness());
+        buffer
+            .truncate(data.bit_length)
+            .map_err(de::Error::custom)?;
+        Ok(BitReadStream::new(buffer))
+    }
+}
+
+#[cfg(feature = "serde")]
+#[test]
+fn test_serde_roundtrip() {
+    use crate::LittleEndian;
+
+    let mut buffer = BitReadBuffer::new_owned(vec![55; 8], LittleEndian);
+    buffer.truncate(61).unwrap();
+    let stream = BitReadStream::new(buffer);
+    assert_eq!(61, stream.bit_len());
+
+    let json = serde_json::to_string(&stream).unwrap();
+
+    let result: BitReadStream<LittleEndian> = serde_json::from_str(&json).unwrap();
+
+    assert_eq!(result, stream);
+}
